@@ -43,9 +43,6 @@
 
 /*** DOCUMENTATION
 	<application name="Directory" language="en_US">
-		<since>
-			<version>0.1.0</version>
-		</since>
 		<synopsis>
 			Provide directory of voicemail extensions.
 		</synopsis>
@@ -358,9 +355,7 @@ static int compare(const char *text, const char *template)
 
 static int goto_exten(struct ast_channel *chan, const char *dialcontext, char *ext)
 {
-	if (!ast_goto_if_exists(chan, S_OR(dialcontext, ast_channel_context(chan)), ext, 1) ||
-		(!ast_strlen_zero(ast_channel_macrocontext(chan)) &&
-		!ast_goto_if_exists(chan, ast_channel_macrocontext(chan), ext, 1))) {
+	if (!ast_goto_if_exists(chan, S_OR(dialcontext, ast_channel_context(chan)), ext, 1)) {
 		return 0;
 	} else {
 		ast_log(LOG_WARNING, "Can't find extension '%s' in current context.  "
