@@ -68,9 +68,6 @@
 
 /*** DOCUMENTATION
 	<application name="FollowMe" language="en_US">
-		<since>
-			<version>1.6.1.0</version>
-		</since>
 		<synopsis>
 			Find-Me/Follow-Me application.
 		</synopsis>
@@ -1534,8 +1531,7 @@ static int app_exec(struct ast_channel *chan, const char *data)
 
 		/* Update connected line to caller if available. */
 		if (targs->pending_out_connected_update) {
-			if (ast_channel_connected_line_sub(outbound, caller, &targs->connected_out, 0) &&
-				ast_channel_connected_line_macro(outbound, caller, &targs->connected_out, 1, 0)) {
+			if (ast_channel_connected_line_sub(outbound, caller, &targs->connected_out, 0)) {
 				ast_channel_update_connected_line(caller, &targs->connected_out, NULL);
 			}
 		}
@@ -1560,8 +1556,7 @@ static int app_exec(struct ast_channel *chan, const char *data)
 
 		/* Update connected line to winner if changed. */
 		if (targs->pending_in_connected_update) {
-			if (ast_channel_connected_line_sub(caller, outbound, &targs->connected_in, 0) &&
-				ast_channel_connected_line_macro(caller, outbound, &targs->connected_in, 0, 0)) {
+			if (ast_channel_connected_line_sub(caller, outbound, &targs->connected_in, 0)) {
 				ast_channel_update_connected_line(outbound, &targs->connected_in, NULL);
 			}
 		}
