@@ -63,9 +63,6 @@
 
 /*** DOCUMENTATION
 	<application name="MixMonitor" language="en_US">
-		<since>
-			<version>1.2.0</version>
-		</since>
 		<synopsis>
 			Record a call and mix the audio during the recording.  Use of StopMixMonitor is required
 			to guarantee the audio file is available for processing during dialplan execution.
@@ -130,13 +127,6 @@
 						Like with the basic filename argument, if an absolute path isn't given, it will create
 						the file in the configured monitoring directory.</para>
 					</option>
-					<option name="D">
-						<para>Interleave the audio coming from the channel and the audio
-						going to the channel and output it as a 2 channel (stereo)
-						raw stream rather than mixing it. You must use the
-						<literal>.raw</literal> file extension. Any other extension
-						will produce a corrupted file.</para>
-					</option>
 					<option name="n">
 						<para>When the <replaceable>r</replaceable> or <replaceable>t</replaceable> option is
 						used, MixMonitor will insert silence into the specified files to maintain
@@ -158,13 +148,6 @@
 						separated by commas eg. m(1111@default,2222@default,...).  Folders can be optionally specified using
 						the syntax: mailbox@context/folder</para>
 					</option>
-					<option name="s">
-						<argument name="seconds" required="true" />
-						<para>Don't record until <replaceable>seconds</replaceable> (can be fractional) have elapsed since MixMonitor was invoked.
-						No audio is written to the recording file during this time. If the call ends before this period,
-						no audio will be saved. This can be useful to avoid recording announcements,
-						ringback tones, or other non-essential early audio.</para>
-					</option>
 				</optionlist>
 			</parameter>
 			<parameter name="command">
@@ -180,7 +163,7 @@
 		</syntax>
 		<description>
 			<para>Records the audio on the current channel to the specified file.</para>
-			<para>This application does not automatically answer and should be preceded by
+			<para>This application does not automatically answer and should be preceeded by
 			an application such as Answer or Progress().</para>
 			<note><para>MixMonitor runs as an audiohook.</para></note>
 			<note><para>If a filename passed to MixMonitor ends with
@@ -198,21 +181,12 @@
 			parameters.  You risk a command injection attack executing arbitrary commands
 			if the untrusted strings aren't filtered to remove dangerous characters.  See
 			function <variable>FILTER()</variable>.</para></warning>
-			<warning><para>When using the <literal>D</literal> option to save
-			interleaved audio, you MUST use <literal>.raw</literal> as the
-			file extension.  Any other extension will produce a corrupted file.</para></warning>
 		</description>
 		<see-also>
-			<ref type="application">Monitor</ref>
 			<ref type="application">StopMixMonitor</ref>
-			<ref type="application">PauseMonitor</ref>
-			<ref type="application">UnpauseMonitor</ref>
 		</see-also>
 	</application>
 	<application name="StopMixMonitor" language="en_US">
-		<since>
-			<version>1.4.0</version>
-		</since>
 		<synopsis>
 			Stop recording a call through MixMonitor, and free the recording's file handle.
 		</synopsis>
@@ -231,9 +205,6 @@
 		</see-also>
 	</application>
 	<manager name="MixMonitorMute" language="en_US">
-		<since>
-			<version>1.8.0</version>
-		</since>
 		<synopsis>
 			Mute / unMute a Mixmonitor recording.
 		</synopsis>
@@ -254,9 +225,6 @@
 		</description>
 	</manager>
 	<manager name="MixMonitor" language="en_US">
-		<since>
-			<version>11.0.0</version>
-		</since>
 		<synopsis>
 			Record a call and mix the audio during the recording.  Use of StopMixMonitor is required
 			to guarantee the audio file is available for processing during dialplan execution.
@@ -274,7 +242,7 @@
 				neither MIXMONITOR_FILENAME or this parameter is set, the mixed stream won't
 				be recorded.</para>
 			</parameter>
-			<parameter name="Options">
+			<parameter name="options">
 				<para>Options that apply to the MixMonitor in the same way as they
 				would apply if invoked from the MixMonitor application. For a list of
 				available options, see the documentation for the mixmonitor application. </para>
@@ -300,9 +268,6 @@
 		</description>
 	</manager>
 	<manager name="StopMixMonitor" language="en_US">
-		<since>
-			<version>11.0.0</version>
-		</since>
 		<synopsis>
 			Stop recording a call through MixMonitor, and free the recording's file handle.
 		</synopsis>
@@ -322,9 +287,6 @@
 		</description>
 	</manager>
 	<function name="MIXMONITOR" language="en_US">
-		<since>
-			<version>13.0.0</version>
-		</since>
 		<synopsis>
 			Retrieve data pertaining to specific instances of MixMonitor on a channel.
 		</synopsis>
@@ -343,10 +305,6 @@
 	</function>
 	<managerEvent language="en_US" name="MixMonitorStart">
 		<managerEventInstance class="EVENT_FLAG_CALL">
-			<since>
-				<version>16.17.0</version>
-				<version>18.3.0</version>
-			</since>
 			<synopsis>Raised when monitoring has started on a channel.</synopsis>
 			<syntax>
 				<channel_snapshot/>
@@ -360,10 +318,6 @@
 	</managerEvent>
 	<managerEvent language="en_US" name="MixMonitorStop">
 		<managerEventInstance class="EVENT_FLAG_CALL">
-			<since>
-				<version>16.17.0</version>
-				<version>18.3.0</version>
-			</since>
 		<synopsis>Raised when monitoring has stopped on a channel.</synopsis>
 		<syntax>
 			<channel_snapshot/>
@@ -377,10 +331,6 @@
 	</managerEvent>
 	<managerEvent language="en_US" name="MixMonitorMute">
 		<managerEventInstance class="EVENT_FLAG_CALL">
-			<since>
-				<version>16.17.0</version>
-				<version>18.3.0</version>
-			</since>
 		<synopsis>Raised when monitoring is muted or unmuted on a channel.</synopsis>
 		<syntax>
 			<channel_snapshot/>
@@ -403,8 +353,6 @@
 
 #define get_volfactor(x) x ? ((x > 0) ? (1 << x) : ((1 << abs(x)) * -1)) : 0
 
-#define MIN_SKIP_SECONDS 1
-
 static const char * const app = "MixMonitor";
 
 static const char * const stop_app = "StopMixMonitor";
@@ -437,16 +385,12 @@ struct mixmonitor {
 	/* the below string fields describe data used for creating voicemails from the recording */
 	AST_DECLARE_STRING_FIELDS(
 		AST_STRING_FIELD(call_context);
-		AST_STRING_FIELD(call_macrocontext);
 		AST_STRING_FIELD(call_extension);
 		AST_STRING_FIELD(call_callerchan);
 		AST_STRING_FIELD(call_callerid);
 	);
 	int call_priority;
 
-	/* Number of seconds (can be fractional) to skip at the start of recording */
-	double skip_seconds;
-
 	/* FUTURE DEVELOPMENT NOTICE
 	 * recipient_list will need locks if we make it editable after the monitor is started */
 	AST_LIST_HEAD_NOLOCK(, vm_recipient) recipient_list;
@@ -470,8 +414,6 @@ enum mixmonitor_flags {
 	MUXFLAG_NO_RWSYNC = (1 << 15),
 	MUXFLAG_AUTO_DELETE = (1 << 16),
 	MUXFLAG_REAL_CALLERID = (1 << 17),
-	MUXFLAG_INTERLEAVED = (1 << 18),
-	MUXFLAG_SKIP = (1 << 19),
 };
 
 enum mixmonitor_args {
@@ -485,7 +427,6 @@ enum mixmonitor_args {
 	OPT_ARG_BEEP_INTERVAL,
 	OPT_ARG_DEPRECATED_RWSYNC,
 	OPT_ARG_NO_RWSYNC,
-	OPT_ARG_SKIP,
 	OPT_ARG_ARRAY_SIZE,	/* Always last element of the enum */
 };
 
@@ -502,12 +443,10 @@ AST_APP_OPTIONS(mixmonitor_opts, {
 	AST_APP_OPTION_ARG('W', MUXFLAG_VOLUME, OPT_ARG_VOLUME),
 	AST_APP_OPTION_ARG('r', MUXFLAG_READ, OPT_ARG_READNAME),
 	AST_APP_OPTION_ARG('t', MUXFLAG_WRITE, OPT_ARG_WRITENAME),
-	AST_APP_OPTION('D', MUXFLAG_INTERLEAVED),
 	AST_APP_OPTION_ARG('i', MUXFLAG_UID, OPT_ARG_UID),
 	AST_APP_OPTION_ARG('m', MUXFLAG_VMRECIPIENTS, OPT_ARG_VMRECIPIENTS),
 	AST_APP_OPTION_ARG('S', MUXFLAG_DEPRECATED_RWSYNC, OPT_ARG_DEPRECATED_RWSYNC),
 	AST_APP_OPTION_ARG('n', MUXFLAG_NO_RWSYNC, OPT_ARG_NO_RWSYNC),
-	AST_APP_OPTION_ARG('s', MUXFLAG_SKIP, OPT_ARG_SKIP),
 });
 
 struct mixmonitor_ds {
@@ -716,7 +655,6 @@ static void copy_to_voicemail(struct mixmonitor *mixmonitor, const char *ext, co
 	ast_string_field_set(&recording_data, recording_file, filename);
 	ast_string_field_set(&recording_data, recording_ext, ext);
 	ast_string_field_set(&recording_data, call_context, mixmonitor->call_context);
-	ast_string_field_set(&recording_data, call_macrocontext, mixmonitor->call_macrocontext);
 	ast_string_field_set(&recording_data, call_extension, mixmonitor->call_extension);
 	ast_string_field_set(&recording_data, call_callerchan, mixmonitor->call_callerchan);
 	ast_string_field_set(&recording_data, call_callerid, mixmonitor->call_callerid);
@@ -792,8 +730,6 @@ static void *mixmonitor_thread(void *obj)
 	int errflag = 0;
 	struct ast_format *format_slin;
 
-	struct timeval skip_start = ast_tvnow();
-
 	/* Keep callid association before any log messages */
 	if (mixmonitor->callid) {
 		ast_callid_threadassoc_add(mixmonitor->callid);
@@ -814,11 +750,6 @@ static void *mixmonitor_thread(void *obj)
 
 	ast_mutex_unlock(&mixmonitor->mixmonitor_ds->lock);
 
-	if (mixmonitor->skip_seconds > 0.0) {
-		ast_debug(3, "%s skipping initial %.3f seconds\n",
-			mixmonitor->name, mixmonitor->skip_seconds);
-	}
-
 	/* The audiohook must enter and exit the loop locked */
 	ast_audiohook_lock(&mixmonitor->audiohook);
 	while (mixmonitor->audiohook.status == AST_AUDIOHOOK_STATUS_RUNNING && !mixmonitor->mixmonitor_ds->fs_quit) {
@@ -844,22 +775,6 @@ static void *mixmonitor_thread(void *obj)
 			|| mixmonitor_autochan_is_bridged(mixmonitor->autochan)) {
 			ast_mutex_lock(&mixmonitor->mixmonitor_ds->lock);
 
-			/* Skip writing audio for the first N seconds */
-			if (mixmonitor->skip_seconds > 0.0) {
-				struct timeval now = ast_tvnow();
-				double elapsed = ast_tvdiff_ms(now, skip_start) / 1000.0;
-
-				if (elapsed < mixmonitor->skip_seconds) {
-					ast_mutex_unlock(&mixmonitor->mixmonitor_ds->lock);
-					/* Skip this frame and continue */
-					goto frame_cleanup;
-				} else {
-					ast_debug(3, "%s skip period %.3f seconds elapsed; starting to write audio\n",
-						mixmonitor->name, mixmonitor->skip_seconds);
-					mixmonitor->skip_seconds = 0.0;
-				}
-			}
-
 			/* Write out the frame(s) */
 			if ((*fs_read) && (fr_read)) {
 				struct ast_frame *cur;
@@ -877,46 +792,6 @@ static void *mixmonitor_thread(void *obj)
 				}
 			}
 
-			if (ast_test_flag(mixmonitor, MUXFLAG_INTERLEAVED)) {
-				/* The 'D' option is set, so mix the frame as an interleaved dual channel frame */
-				int i;
-				short read_buf[SAMPLES_PER_FRAME];
-				short write_buf[SAMPLES_PER_FRAME];
-				short stereo_buf[SAMPLES_PER_FRAME * 2];
-				struct ast_frame stereo_frame = {
-					.frametype = AST_FRAME_VOICE,
-					.datalen = sizeof(stereo_buf),
-					.samples = SAMPLES_PER_FRAME,
-				};
-
-				if (fr) {
-					ast_frame_free(fr, 0);
-					fr = NULL;
-				}
-
-				if (fr_read) {
-					memcpy(read_buf, fr_read->data.ptr, sizeof(read_buf));
-				} else {
-					memset(read_buf, 0, sizeof(read_buf));
-				}
-
-				if (fr_write) {
-					memcpy(write_buf, fr_write->data.ptr, sizeof(write_buf));
-				} else {
-					memset(write_buf, 0, sizeof(write_buf));
-				}
-
-				for (i = 0; i < SAMPLES_PER_FRAME; i++) {
-					stereo_buf[i * 2] = read_buf[i];
-					stereo_buf[i * 2 + 1] = write_buf[i];
-				}
-
-				stereo_frame.data.ptr = stereo_buf;
-				stereo_frame.subclass.format = ast_format_cache_get_slin_by_rate(SAMPLES_PER_FRAME);
-
-				fr = ast_frdup(&stereo_frame);
-			}
-
 			if ((*fs) && (fr)) {
 				struct ast_frame *cur;
 
@@ -926,8 +801,6 @@ static void *mixmonitor_thread(void *obj)
 			}
 			ast_mutex_unlock(&mixmonitor->mixmonitor_ds->lock);
 		}
-
-frame_cleanup:
 		/* All done! free it. */
 		if (fr) {
 			ast_frame_free(fr, 0);
@@ -1069,10 +942,12 @@ static void mixmonitor_ds_remove_and_free(struct ast_channel *chan, const char *
 
 	datastore = ast_channel_datastore_find(chan, &mixmonitor_ds_info, datastore_id);
 
-	if (datastore) {
-		ast_channel_datastore_remove(chan, datastore);
-		ast_datastore_free(datastore);
-	}
+	/*
+	 * Currently the one place this function is called from guarantees a
+	 * datastore is present, thus return checks can be avoided here.
+	 */
+	ast_channel_datastore_remove(chan, datastore);
+	ast_datastore_free(datastore);
 
 	ast_channel_unlock(chan);
 }
@@ -1081,7 +956,7 @@ static int launch_monitor_thread(struct ast_channel *chan, const char *filename,
 				  unsigned int flags, int readvol, int writevol,
 				  const char *post_process, const char *filename_write,
 				  char *filename_read, const char *uid_channel_var,
-				  const char *recipients, const char *beep_id, double skip_seconds)
+				  const char *recipients, const char *beep_id)
 {
 	pthread_t thread;
 	struct mixmonitor *mixmonitor;
@@ -1123,7 +998,6 @@ static int launch_monitor_thread(struct ast_channel *chan, const char *filename,
 
 	/* Copy over flags and channel name */
 	mixmonitor->flags = flags;
-	mixmonitor->skip_seconds = skip_seconds;
 	if (!(mixmonitor->autochan = ast_autochan_setup(chan))) {
 		mixmonitor_free(mixmonitor);
 		return -1;
@@ -1195,7 +1069,6 @@ static int launch_monitor_thread(struct ast_channel *chan, const char *filename,
 		}
 
 		ast_string_field_set(mixmonitor, call_context, ast_channel_context(chan));
-		ast_string_field_set(mixmonitor, call_macrocontext, ast_channel_macrocontext(chan));
 		ast_string_field_set(mixmonitor, call_extension, ast_channel_exten(chan));
 		ast_string_field_set(mixmonitor, call_callerchan, ast_channel_name(chan));
 		ast_string_field_set(mixmonitor, call_callerid, callerid);
@@ -1280,7 +1153,6 @@ static char *filename_parse(char *filename, char *buffer, size_t len)
 static int mixmonitor_exec(struct ast_channel *chan, const char *data)
 {
 	int x, readvol = 0, writevol = 0;
-	double skip_seconds = 0.0;
 	char *filename_read = NULL;
 	char *filename_write = NULL;
 	char filename_buffer[1024] = "";
@@ -1380,22 +1252,6 @@ static int mixmonitor_exec(struct ast_channel *chan, const char *data)
 				return -1;
 			}
 		}
-
-		if (ast_test_flag(&flags, MUXFLAG_SKIP)) {
-			if (ast_strlen_zero(opts[OPT_ARG_SKIP])) {
-				ast_log(LOG_WARNING, "No skip value provided for the 's' (skip) option; skipping will be ignored as no default exists.\n");
-			} else {
-				char *endptr = NULL;
-				double val = strtod(opts[OPT_ARG_SKIP], &endptr);
-				if (endptr == opts[OPT_ARG_SKIP] || *endptr != '\0') {
-					ast_log(LOG_WARNING, "Skip value '%s' is not a valid number; ignoring skip.\n", opts[OPT_ARG_SKIP]);
-				} else if (val < (double) MIN_SKIP_SECONDS) {
-					ast_log(LOG_WARNING, "Skip value %.3f is below minimum %d; ignoring skip.\n", val, MIN_SKIP_SECONDS);
-				} else {
-					skip_seconds = val;
-				}
-			}
-		}
 	}
 	/* If there are no file writing arguments/options for the mix monitor, send a warning message and return -1 */
 
@@ -1423,8 +1279,7 @@ static int mixmonitor_exec(struct ast_channel *chan, const char *data)
 			filename_read,
 			uid_channel_var,
 			recipients,
-			beep_id,
-			skip_seconds)) {
+			beep_id)) {
 		ast_module_unref(ast_module_info->self);
 	}
 
