@@ -28,51 +28,153 @@
  * @{
  */
 
+/*!
+ * \since 17
+ * \brief Channel snapshot invalidation flags, used to force generation of segments
+ */
+enum ast_channel_snapshot_segment_invalidation {
+	/*! Invalidate the bridge segment */
+	AST_CHANNEL_SNAPSHOT_INVALIDATE_BRIDGE = (1 << 1),
+	/*! Invalidate the dialplan segment */
+	AST_CHANNEL_SNAPSHOT_INVALIDATE_DIALPLAN = (1 << 2),
+	/*! Invalidate the connected segment */
+	AST_CHANNEL_SNAPSHOT_INVALIDATE_CONNECTED = (1 << 3),
+	/*! Invalidate the caller segment */
+	AST_CHANNEL_SNAPSHOT_INVALIDATE_CALLER = (1 << 4),
+	/*! Invalidate the hangup segment */
+	AST_CHANNEL_SNAPSHOT_INVALIDATE_HANGUP = (1 << 5),
+	/*! Invalidate the peer segment */
+	AST_CHANNEL_SNAPSHOT_INVALIDATE_PEER = (1 << 6),
+	/*! Invalidate the base segment */
+	AST_CHANNEL_SNAPSHOT_INVALIDATE_BASE = (1 << 7),
+};
+
+/*!
+ * \since 17
+ * \brief Structure containing bridge information for a channel snapshot.
+ */
+struct ast_channel_snapshot_bridge {
+	char id[0]; /*!< Unique Bridge Identifier */
+};
+
+/*!
+ * \since 17
+ * \brief Structure containing dialplan information for a channel snapshot.
+ */
+struct ast_channel_snapshot_dialplan {
+	AST_DECLARE_STRING_FIELDS(
+		AST_STRING_FIELD(appl);             /*!< Current application */
+		AST_STRING_FIELD(data);             /*!< Data passed to current application */
+		AST_STRING_FIELD(context);          /*!< Current extension context */
+		AST_STRING_FIELD(exten);            /*!< Current extension number */
+	);
+	int priority; /*!< Current extension priority */
+};
+
+/*!
+ * \since 17
+ * \brief Structure containing caller information for a channel snapshot.
+ */
+struct ast_channel_snapshot_caller {
+	AST_DECLARE_STRING_FIELDS(
+		AST_STRING_FIELD(name);           /*!< Caller ID Name */
+		AST_STRING_FIELD(number);         /*!< Caller ID Number */
+		AST_STRING_FIELD(dnid);           /*!< Dialed ID Number */
+		AST_STRING_FIELD(dialed_subaddr); /*!< Dialed subaddress */
+		AST_STRING_FIELD(ani);            /*!< Caller ID ANI Number */
+		AST_STRING_FIELD(rdnis);          /*!< Caller ID RDNIS Number */
+		AST_STRING_FIELD(subaddr);        /*!< Caller subaddress */
+	);
+	int pres; /*!< Caller ID presentation. */
+};
+
+/*!
+ * \since 17
+ * \brief Structure containing connected information for a channel snapshot.
+ */
+struct ast_channel_snapshot_connected {
+	char *number; /*!< Connected Line Number */
+	char name[0]; /*!< Connected Line Name */
+};
+
+/*!
+ * \since 17
+ * \brief Structure containing base information for a channel snapshot.
+ */
+struct ast_channel_snapshot_base {
+	AST_DECLARE_STRING_FIELDS(
+		AST_STRING_FIELD(name);        /*!< ASCII unique channel name */
+		AST_STRING_FIELD(uniqueid);    /*!< Unique Channel Identifier */
+		AST_STRING_FIELD(accountcode); /*!< Account code for billing */
+		AST_STRING_FIELD(userfield);   /*!< Userfield for CEL billing */
+		AST_STRING_FIELD(language);    /*!< The default spoken language for the channel */
+		AST_STRING_FIELD(type);        /*!< Type of channel technology */
+		AST_STRING_FIELD(tenantid);    /*!< Channel tenant identifier */
+	);
+	struct timeval creationtime; /*!< The time of channel creation */
+	int tech_properties;         /*!< Properties of the channel's technology */
+	AST_STRING_FIELD_EXTENDED(protocol_id); /*!< Channel driver protocol id (i.e. Call-ID for chan_pjsip) */
+};
+
+/*!
+ * \since 17
+ * \brief Structure containing peer information for a channel snapshot.
+ */
+struct ast_channel_snapshot_peer {
+	char *linkedid;   /*!< Linked Channel Identifier -- gets propagated by linkage */
+	char account[0]; /*!< Peer account code for billing */
+};
+
+/*!
+ * \since 17
+ * \brief Structure containing hangup information for a channel snapshot.
+ */
+struct ast_channel_snapshot_hangup {
+	int cause;      /*!< Why is the channel hanged up. See causes.h */
+	char source[0]; /*!< Who is responsible for hanging up this channel */
+};
+
 /*!
  * \since 12
  * \brief Structure representing a snapshot of channel state.
  *
  * While not enforced programmatically, this object is shared across multiple
  * threads, and should be treated as an immutable object.
+ *
+ * It is guaranteed that the segments of this snapshot will always exist
+ * when accessing the snapshot.
  */
 struct ast_channel_snapshot {
-	AST_DECLARE_STRING_FIELDS(
-		AST_STRING_FIELD(name);             /*!< ASCII unique channel name */
-		AST_STRING_FIELD(uniqueid);         /*!< Unique Channel Identifier */
-		AST_STRING_FIELD(linkedid);         /*!< Linked Channel Identifier -- gets propagated by linkage */
-		AST_STRING_FIELD(appl);             /*!< Current application */
-		AST_STRING_FIELD(data);             /*!< Data passed to current application */
-		AST_STRING_FIELD(context);          /*!< Dialplan: Current extension context */
-		AST_STRING_FIELD(exten);            /*!< Dialplan: Current extension number */
-		AST_STRING_FIELD(accountcode);      /*!< Account code for billing */
-		AST_STRING_FIELD(peeraccount);      /*!< Peer account code for billing */
-		AST_STRING_FIELD(userfield);        /*!< Userfield for CEL billing */
-		AST_STRING_FIELD(hangupsource);     /*!< Who is responsible for hanging up this channel */
-		AST_STRING_FIELD(caller_name);      /*!< Caller ID Name */
-		AST_STRING_FIELD(caller_number);    /*!< Caller ID Number */
-		AST_STRING_FIELD(caller_dnid);      /*!< Dialed ID Number */
-		AST_STRING_FIELD(caller_ani);       /*!< Caller ID ANI Number */
-		AST_STRING_FIELD(caller_rdnis);     /*!< Caller ID RDNIS Number */
-		AST_STRING_FIELD(caller_subaddr);   /*!< Caller subaddress */
-		AST_STRING_FIELD(dialed_subaddr);   /*!< Dialed subaddress */
-		AST_STRING_FIELD(connected_name);   /*!< Connected Line Name */
-		AST_STRING_FIELD(connected_number); /*!< Connected Line Number */
-		AST_STRING_FIELD(language);         /*!< The default spoken language for the channel */
-		AST_STRING_FIELD(bridgeid);         /*!< Unique Bridge Identifier */
-		AST_STRING_FIELD(type);             /*!< Type of channel technology */
-	);
+	struct ast_channel_snapshot_base *base;           /*!< Base information about the channel */
+	struct ast_channel_snapshot_peer *peer;           /*!< Peer information */
+	struct ast_channel_snapshot_caller *caller;       /*!< Information about the caller */
+	struct ast_channel_snapshot_connected *connected; /*!< Information about who this channel is connected to */
+	struct ast_channel_snapshot_bridge *bridge;       /*!< Information about the bridge */
+	struct ast_channel_snapshot_dialplan *dialplan;   /*!< Information about the dialplan */
+	struct ast_channel_snapshot_hangup *hangup;       /*!< Hangup information */
+	enum ast_channel_state state;                     /*!< State of line */
+	int amaflags;                                     /*!< AMA flags for billing */
+	struct ast_flags flags;                           /*!< channel flags of AST_FLAG_ type */
+	struct ast_flags softhangup_flags;                /*!< softhangup channel flags */
+	struct varshead *manager_vars;                    /*!< Variables to be appended to manager events */
+	struct varshead *ari_vars;                        /*!< Variables to be appended to ARI events */
+};
 
-	struct timeval creationtime;            /*!< The time of channel creation */
-	enum ast_channel_state state;           /*!< State of line */
-	int priority;                           /*!< Dialplan: Current extension priority */
-	int amaflags;                           /*!< AMA flags for billing */
-	int hangupcause;                        /*!< Why is the channel hanged up. See causes.h */
-	int caller_pres;                        /*!< Caller ID presentation. */
-	struct ast_flags flags;                 /*!< channel flags of AST_FLAG_ type */
-	struct ast_flags softhangup_flags;      /*!< softhangup channel flags */
-	struct varshead *manager_vars;          /*!< Variables to be appended to manager events */
-	int tech_properties;                    /*!< Properties of the channel's technology */
-	struct varshead *ari_vars;              /*!< Variables to be appended to ARI events */
+/*!
+ * \since 17
+ * \brief Structure representing a change of snapshot of channel state.
+ *
+ * While not enforced programmatically, this object is shared across multiple
+ * threads, and should be treated as an immutable object.
+ *
+ * \note This structure will not have a transition of an old snapshot with no
+ * new snapshot to indicate that a channel has gone away. A new snapshot will
+ * always exist and a channel going away can be determined by checking for the
+ * AST_FLAG_DEAD flag on the new snapshot.
+ */
+struct ast_channel_snapshot_update {
+	struct ast_channel_snapshot *old_snapshot; /*!< The old channel snapshot */
+	struct ast_channel_snapshot *new_snapshot; /*!< The new channel snapshot */
 };
 
 /*!
@@ -94,45 +196,28 @@ struct ast_channel_blob {
  */
 struct ast_multi_channel_blob;
 
-struct stasis_cp_all *ast_channel_cache_all(void);
+struct ao2_container *ast_channel_cache_all(void);
 
 /*!
  * \since 12
  * \brief A topic which publishes the events for all channels.
- * \retval Topic for all channel events.
+ * \return Topic for all channel events.
  */
 struct stasis_topic *ast_channel_topic_all(void);
 
-/*!
- * \since 12
- * \brief A caching topic which caches \ref ast_channel_snapshot messages from
- * ast_channel_events_all(void).
- *
- * \retval Topic for all channel events.
- */
-struct stasis_topic *ast_channel_topic_all_cached(void);
-
-/*!
- * \since 12
- * \brief Primary channel cache, indexed by Uniqueid.
- *
- * \retval Cache of \ref ast_channel_snapshot.
- */
-struct stasis_cache *ast_channel_cache(void);
-
 /*!
  * \since 12
  * \brief Secondary channel cache, indexed by name.
  *
- * \retval Cache of \ref ast_channel_snapshot.
+ * \return Cache of \ref ast_channel_snapshot.
  */
-struct stasis_cache *ast_channel_cache_by_name(void);
+struct ao2_container *ast_channel_cache_by_name(void);
 
 /*!
  * \since 12
- * \brief Message type for \ref ast_channel_snapshot.
+ * \brief Message type for \ref ast_channel_snapshot_update.
  *
- * \retval Message type for \ref ast_channel_snapshot.
+ * \return Message type for \ref ast_channel_snapshot_update.
  */
 struct stasis_message_type *ast_channel_snapshot_type(void);
 
@@ -145,7 +230,7 @@ struct stasis_message_type *ast_channel_snapshot_type(void);
  *
  * \param chan The channel from which to generate a snapshot
  *
- * \retval pointer on success (must be unreffed)
+ * \return pointer on success (must be unreffed)
  * \retval NULL on error
  */
 struct ast_channel_snapshot *ast_channel_snapshot_create(
@@ -156,9 +241,9 @@ struct ast_channel_snapshot *ast_channel_snapshot_create(
  * \brief Obtain the latest \ref ast_channel_snapshot from the \ref stasis cache. This is
  * an ao2 object, so use \ref ao2_cleanup() to deallocate.
  *
- * \param unique_id The channel's unique ID
+ * \param uniqueid The channel's unique ID
  *
- * \retval A \ref ast_channel_snapshot on success
+ * \return A \ref ast_channel_snapshot on success
  * \retval NULL on error
  */
 struct ast_channel_snapshot *ast_channel_snapshot_get_latest(const char *uniqueid);
@@ -170,11 +255,23 @@ struct ast_channel_snapshot *ast_channel_snapshot_get_latest(const char *uniquei
  *
  * \param name The channel's name
  *
- * \retval A \ref ast_channel_snapshot on success
+ * \return A \ref ast_channel_snapshot on success
  * \retval NULL on error
  */
 struct ast_channel_snapshot *ast_channel_snapshot_get_latest_by_name(const char *name);
 
+/*!
+ * \since 17
+ * \brief Send the final channel snapshot for a channel, thus removing it from cache
+ *
+ * \pre chan is locked
+ *
+ * \param chan The channel to send the final channel snapshot for
+ *
+ * \note This will also remove the cached snapshot from the channel itself
+ */
+void ast_channel_publish_final_snapshot(struct ast_channel *chan);
+
 /*!
  * \since 12
  * \brief Creates a \ref ast_channel_blob message.
@@ -190,7 +287,7 @@ struct ast_channel_snapshot *ast_channel_snapshot_get_latest_by_name(const char
  *             \c NULL, ast_json_null() is put into the object.
  *
  * \return \ref ast_channel_blob message.
- * \return \c NULL on error
+ * \retval NULL on error
  */
 struct stasis_message *ast_channel_blob_create(struct ast_channel *chan,
 	struct stasis_message_type *type, struct ast_json *blob);
@@ -206,7 +303,7 @@ struct stasis_message *ast_channel_blob_create(struct ast_channel *chan,
  *             \c NULL, ast_json_null() is put into the object.
  *
  * \return \ref ast_channel_blob message.
- * \return \c NULL on error
+ * \retval NULL on error
  */
 struct stasis_message *ast_channel_blob_create_from_cache(
 	const char *uniqueid, struct stasis_message_type *type,
@@ -222,7 +319,7 @@ struct stasis_message *ast_channel_blob_create_from_cache(
  * \param blob The JSON blob that defines the data of this \ref ast_multi_channel_blob
  *
  * \return \ref ast_multi_channel_blob object
- * \return \c NULL on error
+ * \retval NULL on error
 */
 struct ast_multi_channel_blob *ast_multi_channel_blob_create(struct ast_json *blob);
 
@@ -239,7 +336,7 @@ struct ast_multi_channel_blob *ast_multi_channel_blob_create(struct ast_json *bl
  * to retrieve
  * \param role The role associated with the channel snapshot
  *
- * \retval \ref ast_channel_snapshot matching the role on success
+ * \return \ref ast_channel_snapshot matching the role on success
  * \retval NULL on error or not found for the role specified
  */
 struct ast_channel_snapshot *ast_multi_channel_blob_get_channel(
@@ -260,7 +357,7 @@ struct ast_channel_snapshot *ast_multi_channel_blob_get_channel(
  * retrieve
  * \param role The role associated with the channel snapshots
  *
- * \retval A container containing all \ref ast_channel_snapshot objects matching
+ * \return A container containing all \ref ast_channel_snapshot objects matching
  * the role on success.
  * \retval NULL on error or not found for the role specified
  */
@@ -274,7 +371,7 @@ struct ao2_container *ast_multi_channel_blob_get_channels(
  *
  * \param obj Channel blob object.
  * \return Type field value from the blob.
- * \return \c NULL on error.
+ * \retval NULL on error.
  */
 struct ast_json *ast_multi_channel_blob_get_json(struct ast_multi_channel_blob *obj);
 
@@ -303,7 +400,7 @@ void ast_multi_channel_blob_add_channel(struct ast_multi_channel_blob *obj,
  * \param type Type of stasis message.
  * \param blob The blob being published. (NULL if no blob)
  *
- * \return Nothing
+ * \note This will use the current snapshot on the channel and will not generate a new one.
  */
 void ast_channel_publish_blob(struct ast_channel *chan, struct stasis_message_type *type,
 	struct ast_json *blob);
@@ -319,8 +416,6 @@ void ast_channel_publish_blob(struct ast_channel *chan, struct stasis_message_ty
  * \note As this only accesses the uniqueid and topic of the channel - neither of
  * which should ever be changed on a channel anyhow - a channel does not have to
  * be locked when calling this function.
- *
- * \return Nothing
  */
 void ast_channel_publish_cached_blob(struct ast_channel *chan, struct stasis_message_type *type,
 	struct ast_json *blob);
@@ -345,6 +440,18 @@ void ast_channel_stage_snapshot(struct ast_channel *chan);
  */
 void ast_channel_stage_snapshot_done(struct ast_channel *chan);
 
+/*!
+ * \since 17
+ * \brief Invalidate a channel snapshot segment from being reused
+ *
+ * \pre chan is locked
+ *
+ * \param chan Channel to invalidate the segment on.
+ * \param segment The segment to invalidate.
+ */
+void ast_channel_snapshot_invalidate_segment(struct ast_channel *chan,
+	enum ast_channel_snapshot_segment_invalidation segment);
+
 /*!
  * \since 12
  * \brief Publish a \ref ast_channel_snapshot for a channel.
@@ -357,7 +464,7 @@ void ast_channel_publish_snapshot(struct ast_channel *chan);
 
 /*!
  * \since 12
- * \brief Publish a \ref ast_channel_varset for a channel.
+ * \brief Publish a \ref ast_channel_publish_varset for a channel.
  *
  * \pre chan is locked
  *
@@ -372,7 +479,7 @@ void ast_channel_publish_varset(struct ast_channel *chan,
  * \since 12
  * \brief Message type for when a channel dials another channel
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_dial_type(void);
 
@@ -380,7 +487,7 @@ struct stasis_message_type *ast_channel_dial_type(void);
  * \since 12
  * \brief Message type for when a variable is set on a channel.
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_varset_type(void);
 
@@ -388,7 +495,7 @@ struct stasis_message_type *ast_channel_varset_type(void);
  * \since 12
  * \brief Message type for when a hangup is requested on a channel.
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_hangup_request_type(void);
 
@@ -396,7 +503,7 @@ struct stasis_message_type *ast_channel_hangup_request_type(void);
  * \since 16
  * \brief Message type for when a channel is being masqueraded
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_masquerade_type(void);
 
@@ -404,7 +511,7 @@ struct stasis_message_type *ast_channel_masquerade_type(void);
  * \since 12
  * \brief Message type for when DTMF begins on a channel.
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_dtmf_begin_type(void);
 
@@ -412,15 +519,29 @@ struct stasis_message_type *ast_channel_dtmf_begin_type(void);
  * \since 12
  * \brief Message type for when DTMF ends on a channel.
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_dtmf_end_type(void);
 
+/*!
+ * \brief Message type for when a hook flash occurs on a channel.
+ *
+ * \return A stasis message type
+ */
+struct stasis_message_type *ast_channel_flash_type(void);
+
+/*!
+ * \brief Message type for when a wink occurs on a channel.
+ *
+ * \return A stasis message type
+ */
+struct stasis_message_type *ast_channel_wink_type(void);
+
 /*!
  * \since 12
  * \brief Message type for when a channel is placed on hold.
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_hold_type(void);
 
@@ -428,7 +549,7 @@ struct stasis_message_type *ast_channel_hold_type(void);
  * \since 12
  * \brief Message type for when a channel is removed from hold.
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_unhold_type(void);
 
@@ -436,7 +557,7 @@ struct stasis_message_type *ast_channel_unhold_type(void);
  * \since 12
  * \brief Message type for when a channel starts spying on another channel
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_chanspy_start_type(void);
 
@@ -444,7 +565,7 @@ struct stasis_message_type *ast_channel_chanspy_start_type(void);
  * \since 12
  * \brief Message type for when a channel stops spying on another channel
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_chanspy_stop_type(void);
 
@@ -452,7 +573,7 @@ struct stasis_message_type *ast_channel_chanspy_stop_type(void);
  * \since 12
  * \brief Message type for a fax operation
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_fax_type(void);
 
@@ -460,31 +581,15 @@ struct stasis_message_type *ast_channel_fax_type(void);
  * \since 12
  * \brief Message type for hangup handler related actions
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_hangup_handler_type(void);
 
-/*!
- * \since 12
- * \brief Message type for starting monitor on a channel
- *
- * \retval A stasis message type
- */
-struct stasis_message_type *ast_channel_monitor_start_type(void);
-
-/*!
- * \since 12
- * \brief Message type for stopping monitor on a channel
- *
- * \retval A stasis message type
- */
-struct stasis_message_type *ast_channel_monitor_stop_type(void);
-
 /*!
  * \since 18
  * \brief Message type for starting mixmonitor on a channel
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_mixmonitor_start_type(void);
 
@@ -492,7 +597,7 @@ struct stasis_message_type *ast_channel_mixmonitor_start_type(void);
  * \since 18
  * \brief Message type for stopping mixmonitor on a channel
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_mixmonitor_stop_type(void);
 
@@ -500,7 +605,7 @@ struct stasis_message_type *ast_channel_mixmonitor_stop_type(void);
  * \since 18
  * \brief Message type for muting or unmuting mixmonitor on a channel
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_mixmonitor_mute_type(void);
 
@@ -508,7 +613,7 @@ struct stasis_message_type *ast_channel_mixmonitor_mute_type(void);
  * \since 18.0.0
  * \brief Message type for agent login on a channel
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_agent_login_type(void);
 
@@ -516,7 +621,7 @@ struct stasis_message_type *ast_channel_agent_login_type(void);
  * \since 12.0.0
  * \brief Message type for agent logoff on a channel
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_agent_logoff_type(void);
 
@@ -524,7 +629,7 @@ struct stasis_message_type *ast_channel_agent_logoff_type(void);
  * \since 12
  * \brief Message type for starting music on hold on a channel
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_moh_start_type(void);
 
@@ -532,7 +637,7 @@ struct stasis_message_type *ast_channel_moh_start_type(void);
  * \since 12
  * \brief Message type for stopping music on hold on a channel
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_moh_stop_type(void);
 
@@ -540,7 +645,7 @@ struct stasis_message_type *ast_channel_moh_stop_type(void);
  * \since 12.4.0
  * \brief Message type for a channel starting talking
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_talking_start(void);
 
@@ -548,7 +653,7 @@ struct stasis_message_type *ast_channel_talking_start(void);
  * \since 12.4.0
  * \brief Message type for a channel stopping talking
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_channel_talking_stop(void);
 
@@ -589,17 +694,6 @@ void ast_channel_publish_dial_forward(struct ast_channel *caller,
 		const char *dialstatus,
 		const char *forward);
 
-/*!
- * \since 12
- * \brief Publish in the \ref ast_channel_topic a \ref ast_channel_snapshot
- * message indicating a change in channel state
- *
- * \pre chan is locked
- *
- * \param chan The channel whose state has changed
- */
-void ast_publish_channel_state(struct ast_channel *chan);
-
 /*! @} */
 
 /*!
@@ -609,7 +703,7 @@ void ast_publish_channel_state(struct ast_channel *chan);
  * \param sanitize The message sanitizer to use on the snapshot
  *
  * \return JSON object representing channel snapshot.
- * \return \c NULL on error
+ * \retval NULL on error
  */
 struct ast_json *ast_channel_snapshot_to_json(const struct ast_channel_snapshot *snapshot,
 	const struct stasis_message_sanitizer *sanitize);
@@ -621,8 +715,8 @@ struct ast_json *ast_channel_snapshot_to_json(const struct ast_channel_snapshot
  * \param old_snapshot Old snapshot
  * \param new_snapshot New snapshot
  *
- * \return True (non-zero) if context, exten or priority are identical.
- * \return False (zero) if context, exten and priority changed.
+ * \retval True (non-zero) if context, exten or priority are identical.
+ * \retval False (zero) if context, exten and priority changed.
  */
 int ast_channel_snapshot_cep_equal(
 	const struct ast_channel_snapshot *old_snapshot,
@@ -635,8 +729,8 @@ int ast_channel_snapshot_cep_equal(
  * \param old_snapshot Old snapshot
  * \param new_snapshot New snapshot
  *
- * \return True (non-zero) if callerid are identical.
- * \return False (zero) if callerid changed.
+ * \retval True (non-zero) if callerid are identical.
+ * \retval False (zero) if callerid changed.
  */
 int ast_channel_snapshot_caller_id_equal(
 	const struct ast_channel_snapshot *old_snapshot,
@@ -649,8 +743,8 @@ int ast_channel_snapshot_caller_id_equal(
  * \param old_snapshot Old snapshot
  * \param new_snapshot New snapshot
  *
- * \return True (non-zero) if callerid are identical.
- * \return False (zero) if callerid changed.
+ * \retval True (non-zero) if callerid are identical.
+ * \retval False (zero) if callerid changed.
  */
 int ast_channel_snapshot_connected_line_equal(
 	const struct ast_channel_snapshot *old_snapshot,
@@ -658,8 +752,8 @@ int ast_channel_snapshot_connected_line_equal(
 
 /*!
  * \brief Initialize the stasis channel topic and message types
- * \return 0 on success
- * \return Non-zero on error
+ * \retval 0 on success
+ * \retval Non-zero on error
  */
 int ast_stasis_channels_init(void);
 
