@@ -42,9 +42,6 @@
 
 /*** DOCUMENTATION
 	<application name="Answer" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Answer a channel if ringing.
 		</synopsis>
@@ -75,9 +72,6 @@
 		</see-also>
 	</application>
 	<application name="BackGround" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Play an audio file while waiting for digits of an extension to go to.
 		</synopsis>
@@ -144,9 +138,6 @@
 		</see-also>
 	</application>
 	<application name="Busy" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Indicate the Busy condition.
 		</synopsis>
@@ -167,9 +158,6 @@
 		</see-also>
 	</application>
 	<application name="Congestion" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Indicate the Congestion condition.
 		</synopsis>
@@ -190,9 +178,6 @@
 		</see-also>
 	</application>
 	<application name="ExecIfTime" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Conditional application execution based on the current time.
 		</synopsis>
@@ -220,16 +205,13 @@
 		</see-also>
 	</application>
 	<application name="Goto" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Jump to a particular priority, extension, or context.
 		</synopsis>
 		<syntax>
-			<parameter name="context" documentationtype="dialplan_context" />
-			<parameter name="extension" documentationtype="dialplan_extension" />
-			<parameter name="priority" documentationtype="dialplan_priority" required="true" />
+			<parameter name="context" />
+			<parameter name="extensions" />
+			<parameter name="priority" required="true" />
 		</syntax>
 		<description>
 			<para>This application will set the current context, extension, and priority in the channel structure.
@@ -252,13 +234,9 @@
 			<ref type="application">GotoIf</ref>
 			<ref type="application">GotoIfTime</ref>
 			<ref type="application">Gosub</ref>
-			<ref type="application">Macro</ref>
 		</see-also>
 	</application>
 	<application name="GotoIf" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Conditional goto.
 		</synopsis>
@@ -294,13 +272,9 @@
 			<ref type="application">Goto</ref>
 			<ref type="application">GotoIfTime</ref>
 			<ref type="application">GosubIf</ref>
-			<ref type="application">MacroIf</ref>
 		</see-also>
 	</application>
 	<application name="GotoIfTime" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Conditional Goto based on the current time.
 		</synopsis>
@@ -341,32 +315,7 @@
 			<ref type="function">TESTTIME</ref>
 		</see-also>
 	</application>
-	<application name="ImportVar" language="en_US">
-		<synopsis>
-			Import a variable from a channel into a new variable.
-		</synopsis>
-		<syntax argsep="=">
-			<parameter name="newvar" required="true" />
-			<parameter name="vardata" required="true">
-				<argument name="channelname" required="true" />
-				<argument name="variable" required="true" />
-			</parameter>
-		</syntax>
-		<description>
-			<para>This application imports a <replaceable>variable</replaceable> from the specified
-			<replaceable>channel</replaceable> (as opposed to the current one) and stores it as a variable
-			(<replaceable>newvar</replaceable>) in the current channel (the channel that is calling this
-			application). Variables created by this application have the same inheritance properties as those
-			created with the <literal>Set</literal> application.</para>
-		</description>
-		<see-also>
-			<ref type="application">Set</ref>
-		</see-also>
-	</application>
 	<application name="Hangup" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Hang up the calling channel.
 		</synopsis>
@@ -386,9 +335,6 @@
 		</see-also>
 	</application>
 	<application name="Incomplete" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Returns AST_PBX_INCOMPLETE value.
 		</synopsis>
@@ -406,9 +352,6 @@
 		</description>
 	</application>
 	<application name="NoOp" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Do Nothing (No Operation).
 		</synopsis>
@@ -427,9 +370,6 @@
 		</see-also>
 	</application>
 	<application name="Proceeding" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Indicate proceeding.
 		</synopsis>
@@ -439,9 +379,6 @@
 		</description>
 	</application>
 	<application name="Progress" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Indicate progress.
 		</synopsis>
@@ -457,9 +394,6 @@
 		</see-also>
 	</application>
 	<application name="RaiseException" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Handle an exceptional condition.
 		</synopsis>
@@ -475,9 +409,6 @@
 		</see-also>
 	</application>
 	<application name="Ringing" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Indicate ringing tone.
 		</synopsis>
@@ -493,9 +424,6 @@
 		</see-also>
 	</application>
 	<application name="SayAlpha" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Say Alpha.
 		</synopsis>
@@ -507,7 +435,7 @@
 			of the given <replaceable>string</replaceable>. If the channel variable
 			<variable>SAY_DTMF_INTERRUPT</variable> is set to 'true' (case insensitive),
 			then this application will react to DTMF in the	same way as
-			<literal>BackGround</literal>.</para>
+			<literal>Background</literal>.</para>
 		</description>
 		<see-also>
 			<ref type="application">SayDigits</ref>
@@ -520,9 +448,6 @@
 		</see-also>
 	</application>
 	<application name="SayAlphaCase" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Say Alpha.
 		</synopsis>
@@ -554,7 +479,7 @@
 			given <replaceable>string</replaceable>.  Optionally, a <replaceable>casetype</replaceable> may be
 			specified.  This will be used for case-insensitive or case-sensitive pronunciations. If the channel
 			variable <variable>SAY_DTMF_INTERRUPT</variable> is set to 'true' (case insensitive), then this
-			application will react to DTMF in the same way as <literal>BackGround</literal>.</para>
+			application will react to DTMF in the same way as <literal>Background</literal>.</para>
 		</description>
 		<see-also>
 			<ref type="application">SayDigits</ref>
@@ -567,9 +492,6 @@
 		</see-also>
 	</application>
 	<application name="SayDigits" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Say Digits.
 		</synopsis>
@@ -581,7 +503,7 @@
 			the given number. This will use the language that is currently set for the channel.
 			If the channel variable <variable>SAY_DTMF_INTERRUPT</variable> is set to 'true'
 			(case insensitive), then this application will react to DTMF in the same way as
-			<literal>BackGround</literal>.</para>
+			<literal>Background</literal>.</para>
 		</description>
 		<see-also>
 			<ref type="application">SayAlpha</ref>
@@ -594,10 +516,6 @@
 		</see-also>
 	</application>
 	<application name="SayMoney" language="en_US">
-		<since>
-			<version>16.21.0</version>
-			<version>18.7.0</version>
-		</since>
 		<synopsis>
 			Say Money.
 		</synopsis>
@@ -609,7 +527,7 @@
 			in the current language. Currently only English and US Dollars is supported.
 			If the channel variable <variable>SAY_DTMF_INTERRUPT</variable> is set to 'true'
 			(case insensitive), then this application will react to DTMF in the same way as
-			<literal>BackGround</literal>.</para>
+			<literal>Background</literal>.</para>
 		</description>
 		<see-also>
 			<ref type="application">SayAlpha</ref>
@@ -621,9 +539,6 @@
 		</see-also>
 	</application>
 	<application name="SayNumber" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Say Number.
 		</synopsis>
@@ -637,7 +552,7 @@
 			specified. This will use the language that is currently set for the channel. See the CHANNEL()
 			function for more information on setting the language for the channel. If the channel variable
 			<variable>SAY_DTMF_INTERRUPT</variable> is set to 'true' (case insensitive), then this
-			application will react to DTMF in the same way as <literal>BackGround</literal>.</para>
+			application will react to DTMF in the same way as <literal>Background</literal>.</para>
 		</description>
 		<see-also>
 			<ref type="application">SayAlpha</ref>
@@ -649,10 +564,6 @@
 		</see-also>
 	</application>
 	<application name="SayOrdinal" language="en_US">
-		<since>
-			<version>16.21.0</version>
-			<version>18.7.0</version>
-		</since>
 		<synopsis>
 			Say Ordinal Number.
 		</synopsis>
@@ -667,7 +578,7 @@
 			specified. This will use the language that is currently set for the channel. See the CHANNEL()
 			function for more information on setting the language for the channel. If the channel variable
 			<variable>SAY_DTMF_INTERRUPT</variable> is set to 'true' (case insensitive), then this
-			application will react to DTMF in the same way as <literal>BackGround</literal>.</para>
+			application will react to DTMF in the same way as <literal>Background</literal>.</para>
 		</description>
 		<see-also>
 			<ref type="application">SayAlpha</ref>
@@ -680,9 +591,6 @@
 		</see-also>
 	</application>
 	<application name="SayPhonetic" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Say Phonetic.
 		</synopsis>
@@ -693,7 +601,7 @@
 			<para>This application will play the sounds from the phonetic alphabet that correspond to the
 			letters in the given <replaceable>string</replaceable>. If the channel variable
 			<variable>SAY_DTMF_INTERRUPT</variable> is set to 'true' (case insensitive), then this
-			application will react to DTMF in the same way as <literal>BackGround</literal>.</para>
+			application will react to DTMF in the same way as <literal>Background</literal>.</para>
 		</description>
 		<see-also>
 			<ref type="application">SayAlpha</ref>
@@ -704,26 +612,7 @@
 			<ref type="function">SAYFILES</ref>
 		</see-also>
 	</application>
-	<application name="SetAMAFlags" language="en_US">
-		<synopsis>
-			Set the AMA Flags.
-		</synopsis>
-		<syntax>
-			<parameter name="flag" />
-		</syntax>
-		<description>
-			<para>This application will set the channel's AMA Flags for billing purposes.</para>
-			<warning><para>This application is deprecated. Please use the CHANNEL function instead.</para></warning>
-		</description>
-		<see-also>
-			<ref type="function">CDR</ref>
-			<ref type="function">CHANNEL</ref>
-		</see-also>
-	</application>
 	<application name="Wait" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Waits for some time.
 		</synopsis>
@@ -738,9 +627,6 @@
 		</description>
 	</application>
 	<application name="WaitDigit" language="en_US">
-		<since>
-			<version>15.0.0</version>
-		</since>
 		<synopsis>
 			Waits for a digit to be entered.
 		</synopsis>
@@ -777,9 +663,6 @@
 		</see-also>
 	</application>
 	<application name="WaitExten" language="en_US">
-		<since>
-			<version>13.8.0</version>
-		</since>
 		<synopsis>
 			Waits for an extension to be entered.
 		</synopsis>
@@ -798,11 +681,8 @@
 						</argument>
 					</option>
 					<option name="d">
-						<para>Play indications tone on channel while waiting for digits.</para>
-						<argument name="x">
-							<para>Specify the indications tone to play.
-							Default is <literal>dial</literal> tone.</para>
-						</argument>
+						<para>Play <literal>dial</literal> indications tone on channel while waiting
+						for digits.</para>
 					</option>
 				</optionlist>
 			</parameter>
@@ -810,10 +690,9 @@
 		<description>
 			<para>This application waits for the user to enter a new extension for a specified number
 			of <replaceable>seconds</replaceable>.</para>
-			<xi:include xpointer="xpointer(/docs/application[@name='Macro']/description/warning[2])" />
 		</description>
 		<see-also>
-			<ref type="application">BackGround</ref>
+			<ref type="application">Background</ref>
 			<ref type="function">TIMEOUT</ref>
 		</see-also>
 	</application>
@@ -831,21 +710,12 @@ AST_APP_OPTIONS(background_opts, {
 	AST_APP_OPTION('p', BACKGROUND_PLAYBACK),
 });
 
-enum {
-	WAITEXTEN_MOH = (1 << 0),
-	WAITEXTEN_DIALTONE = (1 << 1),
-};
-
-enum read_option_flags {
-	WAITEXTEN_ARG_MOH,
-	WAITEXTEN_ARG_DIALTONE,
-	/* note: this entry _MUST_ be the last one in the enum */
-	WAITEXTEN_ARRAY_SIZE,
-};
+#define WAITEXTEN_MOH		(1 << 0)
+#define WAITEXTEN_DIALTONE	(1 << 1)
 
 AST_APP_OPTIONS(waitexten_opts, {
-	AST_APP_OPTION_ARG('m', WAITEXTEN_MOH, WAITEXTEN_ARG_MOH),
-	AST_APP_OPTION_ARG('d', WAITEXTEN_DIALTONE, WAITEXTEN_ARG_DIALTONE),
+	AST_APP_OPTION_ARG('m', WAITEXTEN_MOH, 0),
+	AST_APP_OPTION_ARG('d', WAITEXTEN_DIALTONE, 0),
 });
 
 int pbx_builtin_raise_exception(struct ast_channel *chan, const char *reason)
@@ -926,7 +796,7 @@ static int pbx_builtin_answer(struct ast_channel *chan, const char *data)
 	char *parse;
 	AST_DECLARE_APP_ARGS(args,
 		AST_APP_ARG(delay);
-		AST_APP_ARG(answer_cdr);
+		AST_APP_ARG(options);
 	);
 
 	if (ast_strlen_zero(data)) {
@@ -944,15 +814,10 @@ static int pbx_builtin_answer(struct ast_channel *chan, const char *data)
 		delay = 0;
 	}
 
-	if (!ast_strlen_zero(args.answer_cdr) && !strcmp(args.answer_cdr, "i")) {
-		/*! \todo We will remove the nocdr stuff for 21 entirely, as part of another review. */
+	if (!ast_strlen_zero(args.options) && !strcmp(args.options, "i")) {
 		return ast_raw_answer(chan);
 	}
 
-	if (!ast_strlen_zero(args.answer_cdr) && !strcasecmp(args.answer_cdr, "nocdr")) {
-		ast_log(AST_LOG_WARNING, "The nocdr option for the Answer application has been removed and is no longer supported.\n");
-	}
-
 	return __ast_answer(chan, delay);
 }
 
@@ -978,34 +843,6 @@ static int pbx_builtin_incomplete(struct ast_channel *chan, const char *data)
 	return AST_PBX_INCOMPLETE;
 }
 
-/*!
- * \ingroup applications
- */
-static int pbx_builtin_setamaflags(struct ast_channel *chan, const char *data)
-{
-	ast_log(AST_LOG_WARNING, "The SetAMAFlags application is deprecated. Please use the CHANNEL function instead.\n");
-
-	if (ast_strlen_zero(data)) {
-		ast_log(AST_LOG_WARNING, "No parameter passed to SetAMAFlags\n");
-		return 0;
-	}
-	/* Copy the AMA Flags as specified */
-	ast_channel_lock(chan);
-	if (isdigit(data[0])) {
-		int amaflags;
-		if (sscanf(data, "%30d", &amaflags) != 1) {
-			ast_log(AST_LOG_WARNING, "Unable to set AMA flags on channel %s\n", ast_channel_name(chan));
-			ast_channel_unlock(chan);
-			return 0;
-		}
-		ast_channel_amaflags_set(chan, amaflags);
-	} else {
-		ast_channel_amaflags_set(chan, ast_channel_string2amaflag(data));
-	}
-	ast_channel_unlock(chan);
-	return 0;
-}
-
 /*!
  * \ingroup applications
  */
@@ -1215,7 +1052,7 @@ static int pbx_builtin_waitexten(struct ast_channel *chan, const char *data)
 {
 	int ms, res;
 	struct ast_flags flags = {0};
-	char *opt_args[WAITEXTEN_ARRAY_SIZE];
+	char *opts[1] = { NULL };
 	char *parse;
 	AST_DECLARE_APP_ARGS(args,
 		AST_APP_ARG(timeout);
@@ -1229,17 +1066,15 @@ static int pbx_builtin_waitexten(struct ast_channel *chan, const char *data)
 		memset(&args, 0, sizeof(args));
 
 	if (args.options)
-		ast_app_parse_options(waitexten_opts, &flags, opt_args, args.options);
+		ast_app_parse_options(waitexten_opts, &flags, opts, args.options);
 
-	if (ast_test_flag(&flags, WAITEXTEN_MOH)) {
-		if (ast_strlen_zero(opt_args[WAITEXTEN_ARG_MOH])) {
-			ast_log(LOG_WARNING, "The 'm' option has been specified for WaitExten without a class.\n");
-		}
-		ast_indicate_data(chan, AST_CONTROL_HOLD, S_OR(opt_args[WAITEXTEN_ARG_MOH], NULL),
-			!ast_strlen_zero(opt_args[WAITEXTEN_ARG_MOH]) ? strlen(opt_args[WAITEXTEN_ARG_MOH]) + 1 : 0);
+	if (ast_test_flag(&flags, WAITEXTEN_MOH) && !opts[0] ) {
+		ast_log(LOG_WARNING, "The 'm' option has been specified for WaitExten without a class.\n");
+	} else if (ast_test_flag(&flags, WAITEXTEN_MOH)) {
+		ast_indicate_data(chan, AST_CONTROL_HOLD, S_OR(opts[0], NULL),
+			!ast_strlen_zero(opts[0]) ? strlen(opts[0]) + 1 : 0);
 	} else if (ast_test_flag(&flags, WAITEXTEN_DIALTONE)) {
-		const char *tone = !ast_strlen_zero(opt_args[WAITEXTEN_ARG_DIALTONE]) ? opt_args[WAITEXTEN_ARG_DIALTONE] : "dial";
-		struct ast_tone_zone_sound *ts = ast_get_indication_tone(ast_channel_zone(chan), tone);
+		struct ast_tone_zone_sound *ts = ast_get_indication_tone(ast_channel_zone(chan), "dial");
 		if (ts) {
 			ast_playtones_start(chan, 0, ts->data, 0);
 			ts = ast_tone_zone_sound_unref(ts);
@@ -1315,13 +1150,8 @@ static int pbx_builtin_background(struct ast_channel *chan, const char *data)
 		args.lang = (char *)ast_channel_language(chan);	/* XXX this is const */
 
 	if (ast_strlen_zero(args.context)) {
-		const char *context;
 		ast_channel_lock(chan);
-		if ((context = pbx_builtin_getvar_helper(chan, "MACRO_CONTEXT"))) {
-			args.context = ast_strdupa(context);
-		} else {
-			args.context = ast_strdupa(ast_channel_context(chan));
-		}
+		args.context = ast_strdupa(ast_channel_context(chan));
 		ast_channel_unlock(chan);
 	}
 
@@ -1377,9 +1207,7 @@ static int pbx_builtin_background(struct ast_channel *chan, const char *data)
 	/*
 	 * If the single digit DTMF is an extension in the specified context, then
 	 * go there and signal no DTMF.  Otherwise, we should exit with that DTMF.
-	 * If we're in Macro, we'll exit and seek that DTMF as the beginning of an
-	 * extension in the Macro's calling context.  If we're not in Macro, then
-	 * we'll simply seek that extension in the calling context.  Previously,
+	 * We'll simply seek that extension in the calling context.  Previously,
 	 * someone complained about the behavior as it related to the interior of a
 	 * Gosub routine, and the fix (#14011) inadvertently broke FreePBX
 	 * (#14940).  This change should fix both of these situations, but with the
@@ -1629,41 +1457,6 @@ static int pbx_builtin_sayphonetic(struct ast_channel *chan, const char *data)
 	return res;
 }
 
-static int pbx_builtin_importvar(struct ast_channel *chan, const char *data)
-{
-	char *name;
-	char *value;
-	char *channel;
-	char tmp[VAR_BUF_SIZE];
-	static int deprecation_warning = 0;
-
-	if (ast_strlen_zero(data)) {
-		ast_log(LOG_WARNING, "Ignoring, since there is no variable to set\n");
-		return 0;
-	}
-	tmp[0] = 0;
-	if (!deprecation_warning) {
-		ast_log(LOG_WARNING, "ImportVar is deprecated.  Please use Set(varname=${IMPORT(channel,variable)}) instead.\n");
-		deprecation_warning = 1;
-	}
-
-	value = ast_strdupa(data);
-	name = strsep(&value,"=");
-	channel = strsep(&value,",");
-	if (channel && value && name) { /*! \todo XXX should do !ast_strlen_zero(..) of the args ? */
-		struct ast_channel *chan2 = ast_channel_get_by_name(channel);
-		if (chan2) {
-			char *s = ast_alloca(strlen(value) + 4);
-			sprintf(s, "${%s}", value);
-			pbx_substitute_variables_helper(chan2, s, tmp, sizeof(tmp) - 1);
-			chan2 = ast_channel_unref(chan2);
-		}
-		pbx_builtin_setvar_helper(chan, name, tmp);
-	}
-
-	return(0);
-}
-
 /*! \brief Declaration of builtin applications */
 struct pbx_builtin {
 	char name[AST_MAX_APP];
@@ -1681,7 +1474,6 @@ struct pbx_builtin {
 	{ "Goto",           pbx_builtin_goto },
 	{ "GotoIf",         pbx_builtin_gotoif },
 	{ "GotoIfTime",     pbx_builtin_gotoiftime },
-	{ "ImportVar",      pbx_builtin_importvar },
 	{ "Hangup",         pbx_builtin_hangup },
 	{ "Incomplete",     pbx_builtin_incomplete },
 	{ "NoOp",           pbx_builtin_noop },
@@ -1696,7 +1488,6 @@ struct pbx_builtin {
 	{ "SayNumber",      pbx_builtin_saynumber },
 	{ "SayOrdinal",     pbx_builtin_sayordinal },
 	{ "SayPhonetic",    pbx_builtin_sayphonetic },
-	{ "SetAMAFlags",    pbx_builtin_setamaflags },
 	{ "Wait",           pbx_builtin_wait },
 	{ "WaitDigit",      pbx_builtin_waitdigit },
 	{ "WaitExten",      pbx_builtin_waitexten }
