@@ -33,7 +33,7 @@
 #include "asterisk/stasis_internal.h"
 #include "asterisk/stasis.h"
 #include "asterisk/taskprocessor.h"
-#include "asterisk/taskpool.h"
+#include "asterisk/threadpool.h"
 #include "asterisk/utils.h"
 #include "asterisk/uuid.h"
 #include "asterisk/vector.h"
@@ -46,9 +46,6 @@
 /*** DOCUMENTATION
 	<managerEvent language="en_US" name="UserEvent">
 		<managerEventInstance class="EVENT_FLAG_USER">
-			<since>
-				<version>12.3.0</version>
-			</since>
 			<synopsis>A user defined event raised from the dialplan.</synopsis>
 			<syntax>
 				<channel_snapshot/>
@@ -68,83 +65,20 @@
 	<configInfo name="stasis" language="en_US">
 		<configFile name="stasis.conf">
 			<configObject name="threadpool">
-				<since>
-					<version>12.8.0</version>
-					<version>13.1.0</version>
-				</since>
 				<synopsis>Settings that configure the threadpool Stasis uses to deliver some messages.</synopsis>
 				<configOption name="initial_size" default="5">
-					<since>
-						<version>12.8.0</version>
-						<version>13.1.0</version>
-					</since>
 					<synopsis>Initial number of threads in the message bus threadpool.</synopsis>
 				</configOption>
 				<configOption name="idle_timeout_sec" default="20">
-					<since>
-						<version>12.8.0</version>
-						<version>13.1.0</version>
-					</since>
 					<synopsis>Number of seconds before an idle thread is disposed of.</synopsis>
 				</configOption>
 				<configOption name="max_size" default="50">
-					<since>
-						<version>12.8.0</version>
-						<version>13.1.0</version>
-					</since>
 					<synopsis>Maximum number of threads in the threadpool.</synopsis>
 				</configOption>
 			</configObject>
-			<configObject name="taskpool">
-				<since>
-					<version>23.1.0</version>
-					<version>22.7.0</version>
-					<version>20.17.0</version>
-				</since>
-				<synopsis>Settings that configure the taskpool Stasis uses to deliver some messages.</synopsis>
-				<configOption name="minimum_size" default="5">
-					<since>
-					<version>23.1.0</version>
-					<version>22.7.0</version>
-					<version>20.17.0</version>
-					</since>
-					<synopsis>Minimum number of taskprocessors in the message bus taskpool.</synopsis>
-				</configOption>
-				<configOption name="initial_size" default="5">
-					<since>
-					<version>23.1.0</version>
-					<version>22.7.0</version>
-					<version>20.17.0</version>
-					</since>
-					<synopsis>Initial number of taskprocessors in the message bus taskpool.</synopsis>
-				</configOption>
-				<configOption name="idle_timeout_sec" default="20">
-					<since>
-					<version>23.1.0</version>
-					<version>22.7.0</version>
-					<version>20.17.0</version>
-					</since>
-					<synopsis>Number of seconds before an idle taskprocessor is disposed of.</synopsis>
-				</configOption>
-				<configOption name="max_size" default="50">
-					<since>
-					<version>23.1.0</version>
-					<version>22.7.0</version>
-					<version>20.17.0</version>
-					</since>
-					<synopsis>Maximum number of taskprocessors in the taskpool.</synopsis>
-				</configOption>
-			</configObject>
 			<configObject name="declined_message_types">
-				<since>
-					<version>13.0.0</version>
-				</since>
 				<synopsis>Stasis message types for which to decline creation.</synopsis>
 				<configOption name="decline">
-					<since>
-						<version>12.8.0</version>
-						<version>13.1.0</version>
-					</since>
 					<synopsis>The message type to decline.</synopsis>
 					<description>
 						<para>This configuration option defines the name of the Stasis
@@ -196,8 +130,6 @@
 							<enum name="ast_channel_hangup_handler_type" />
 							<enum name="ast_channel_moh_start_type" />
 							<enum name="ast_channel_moh_stop_type" />
-							<enum name="ast_channel_monitor_start_type" />
-							<enum name="ast_channel_monitor_stop_type" />
 							<enum name="ast_channel_mixmonitor_start_type" />
 							<enum name="ast_channel_mixmonitor_stop_type" />
 							<enum name="ast_channel_mixmonitor_mute_type" />
@@ -309,7 +241,7 @@
  * subscriptions need the topics to unsubscribe and check subscription status.
  *
  * The cycle is broken by stasis_unsubscribe(). The unsubscribe will remove the
- * topic's reference to a subscription. When the subscription is destroyed, it
+ * topic's reference to a subscription. When the subcription is destroyed, it
  * will remove its reference to the topic.
  *
  * This means that until a subscription has be explicitly unsubscribed, it will
@@ -371,8 +303,8 @@
 /*! The number of buckets to use for topic pools */
 #define TOPIC_POOL_BUCKETS 57
 
-/*! Taskpool for topics that don't want a dedicated taskprocessor */
-static struct ast_taskpool *taskpool;
+/*! Thread pool for topics that don't want a dedicated taskprocessor */
+static struct ast_threadpool *threadpool;
 
 STASIS_MESSAGE_TYPE_DEFN(stasis_subscription_change_type);
 
@@ -733,8 +665,8 @@ struct stasis_subscription_statistics {
 	int messages_passed;
 	/*! \brief Using a mailbox to queue messages */
 	int uses_mailbox;
-	/*! \brief Using stasis taskpool for handling messages */
-	int uses_taskpool;
+	/*! \brief Using stasis threadpool for handling messages */
+	int uses_threadpool;
 	/*! \brief The line number where the subscription originates */
 	int lineno;
 	/*! \brief Pointer to the subscription (NOT refcounted, and must NOT be accessed) */
@@ -887,7 +819,7 @@ static void subscription_statistics_destroy(void *obj)
 }
 
 static struct stasis_subscription_statistics *stasis_subscription_statistics_create(struct stasis_subscription *sub,
-	int needs_mailbox, int use_taskpool, const char *file, int lineno,
+	int needs_mailbox, int use_thread_pool, const char *file, int lineno,
 	const char *func)
 {
 	struct stasis_subscription_statistics *statistics;
@@ -912,7 +844,7 @@ static struct stasis_subscription_statistics *stasis_subscription_statistics_cre
 	statistics->lineno = lineno;
 	statistics->func = func;
 	statistics->uses_mailbox = needs_mailbox;
-	statistics->uses_taskpool = use_taskpool;
+	statistics->uses_threadpool = use_thread_pool;
 	strcpy(statistics->uniqueid, sub->uniqueid); /* SAFE */
 	statistics->sub = sub;
 	ao2_link(subscription_stats, statistics);
@@ -926,7 +858,7 @@ struct stasis_subscription *internal_stasis_subscribe(
 	stasis_subscription_cb callback,
 	void *data,
 	int needs_mailbox,
-	int use_taskpool,
+	int use_thread_pool,
 	const char *file,
 	int lineno,
 	const char *func)
@@ -946,7 +878,7 @@ struct stasis_subscription *internal_stasis_subscribe(
 
 #ifdef AST_DEVMODE
 	ret = ast_asprintf(&sub->uniqueid, "%s:%s-%d", file, stasis_topic_name(topic), ast_atomic_fetchadd_int(&topic->subscriber_id, +1));
-	sub->statistics = stasis_subscription_statistics_create(sub, needs_mailbox, use_taskpool, file, lineno, func);
+	sub->statistics = stasis_subscription_statistics_create(sub, needs_mailbox, use_thread_pool, file, lineno, func);
 	if (ret < 0 || !sub->statistics) {
 		ao2_ref(sub, -1);
 		return NULL;
@@ -964,7 +896,7 @@ struct stasis_subscription *internal_stasis_subscribe(
 
 		/* Create name with seq number appended. */
 		ast_taskprocessor_build_name(tps_name, sizeof(tps_name), "stasis/%c:%s",
-			use_taskpool ? 'p' : 'm',
+			use_thread_pool ? 'p' : 'm',
 			stasis_topic_name(topic));
 
 		/*
@@ -972,8 +904,8 @@ struct stasis_subscription *internal_stasis_subscribe(
 		 * acceptable. For a large number of subscribers, a thread
 		 * pool should be used.
 		 */
-		if (use_taskpool) {
-			sub->mailbox = ast_taskpool_serializer(tps_name, taskpool);
+		if (use_thread_pool) {
+			sub->mailbox = ast_threadpool_serializer(tps_name, threadpool);
 		} else {
 			sub->mailbox = ast_taskprocessor_get(tps_name, TPS_REF_DEFAULT);
 		}
@@ -2250,21 +2182,19 @@ struct stasis_declined_config {
 	struct ao2_container *declined;
 };
 
-/*! \brief Taskpool configuration options */
-struct stasis_taskpool_conf {
-	/*! Minimum size of the taskpool */
-	int minimum_size;
-	/*! Initial size of the taskpool */
+/*! \brief Threadpool configuration options */
+struct stasis_threadpool_conf {
+	/*! Initial size of the thread pool */
 	int initial_size;
-	/*! Time, in seconds, before we expire a taskprocessor */
+	/*! Time, in seconds, before we expire a thread */
 	int idle_timeout_sec;
-	/*! Maximum number of taskprocessors to allow */
+	/*! Maximum number of thread to allow */
 	int max_size;
 };
 
 struct stasis_config {
-	/*! Taskpool configuration options */
-	struct stasis_taskpool_conf *taskpool_options;
+	/*! Thread pool configuration options */
+	struct stasis_threadpool_conf *threadpool_options;
 	/*! Declined message types */
 	struct stasis_declined_config *declined_message_types;
 };
@@ -2272,20 +2202,12 @@ struct stasis_config {
 static struct aco_type threadpool_option = {
 	.type = ACO_GLOBAL,
 	.name = "threadpool",
-	.item_offset = offsetof(struct stasis_config, taskpool_options),
+	.item_offset = offsetof(struct stasis_config, threadpool_options),
 	.category = "threadpool",
 	.category_match = ACO_WHITELIST_EXACT,
 };
 
-static struct aco_type taskpool_option = {
-	.type = ACO_GLOBAL,
-	.name = "taskpool",
-	.item_offset = offsetof(struct stasis_config, taskpool_options),
-	.category = "taskpool",
-	.category_match = ACO_WHITELIST_EXACT,
-};
-
-static struct aco_type *taskpool_options[] = ACO_TYPES(&threadpool_option, &taskpool_option);
+static struct aco_type *threadpool_options[] = ACO_TYPES(&threadpool_option);
 
 /*! \brief An aco_type structure to link the "declined_message_types" category to the stasis_declined_config type */
 static struct aco_type declined_option = {
@@ -2300,7 +2222,7 @@ struct aco_type *declined_options[] = ACO_TYPES(&declined_option);
 
 struct aco_file stasis_conf = {
         .filename = "stasis.conf",
-	.types = ACO_TYPES(&declined_option, &threadpool_option, &taskpool_option),
+	.types = ACO_TYPES(&declined_option, &threadpool_option),
 };
 
 /*! \brief A global object container that will contain the stasis_config that gets swapped out on reloads */
@@ -2325,7 +2247,7 @@ static void stasis_config_destructor(void *obj)
 	struct stasis_config *cfg = obj;
 
 	ao2_cleanup(cfg->declined_message_types);
-	ast_free(cfg->taskpool_options);
+	ast_free(cfg->threadpool_options);
 }
 
 static void *stasis_config_alloc(void)
@@ -2336,8 +2258,8 @@ static void *stasis_config_alloc(void)
 		return NULL;
 	}
 
-	cfg->taskpool_options = ast_calloc(1, sizeof(*cfg->taskpool_options));
-	if (!cfg->taskpool_options) {
+	cfg->threadpool_options = ast_calloc(1, sizeof(*cfg->threadpool_options));
+	if (!cfg->threadpool_options) {
 		ao2_ref(cfg, -1);
 		return NULL;
 	}
@@ -2435,10 +2357,10 @@ static char *stasis_show_topics(struct ast_cli_entry *e, int cmd, struct ast_cli
 
 	ast_cli(a->fd, "\n" FMT_HEADERS, "Name", "Detail");
 
-	tmp_container = ao2_container_alloc_rbtree(AO2_ALLOC_OPT_LOCK_NOLOCK, 0,
+	tmp_container = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_NOLOCK, 0,
 				topic_proxy_sort_fn, NULL);
 
-	if (!tmp_container || ao2_container_dup(tmp_container, topic_all, 0)) {
+	if (!tmp_container || ao2_container_dup(tmp_container, topic_all, OBJ_SEARCH_OBJECT)) {
 		ao2_cleanup(tmp_container);
 
 		return NULL;
@@ -2707,7 +2629,7 @@ static char *statistics_show_subscription(struct ast_cli_entry *e, int cmd, stru
 
 	subscription_stats = ao2_global_obj_ref(subscription_statistics);
 	if (!subscription_stats) {
-		ast_cli(a->fd, "Could not fetch subscription_statistics container\n");
+		ast_cli(a->fd, "Could not fetch subcription_statistics container\n");
 		return CLI_FAILURE;
 	}
 
@@ -2728,7 +2650,7 @@ static char *statistics_show_subscription(struct ast_cli_entry *e, int cmd, stru
 	ast_cli(a->fd, "Number of messages dropped due to filtering: %d\n", statistics->messages_dropped);
 	ast_cli(a->fd, "Number of messages passed to subscriber callback: %d\n", statistics->messages_passed);
 	ast_cli(a->fd, "Using mailbox to queue messages: %s\n", statistics->uses_mailbox ? "Yes" : "No");
-	ast_cli(a->fd, "Using stasis taskpool for handling messages: %s\n", statistics->uses_taskpool ? "Yes" : "No");
+	ast_cli(a->fd, "Using stasis threadpool for handling messages: %s\n", statistics->uses_threadpool ? "Yes" : "No");
 	ast_cli(a->fd, "Lowest amount of time (in milliseconds) spent invoking message: %ld\n", statistics->lowest_time_invoked);
 	ast_cli(a->fd, "Highest amount of time (in milliseconds) spent invoking message: %ld\n", statistics->highest_time_invoked);
 
@@ -3128,8 +3050,8 @@ static void stasis_cleanup(void)
 	ast_cli_unregister_multiple(cli_stasis, ARRAY_LEN(cli_stasis));
 	ao2_cleanup(topic_all);
 	topic_all = NULL;
-	ast_taskpool_shutdown(taskpool);
-	taskpool = NULL;
+	ast_threadpool_shutdown(threadpool);
+	threadpool = NULL;
 	STASIS_MESSAGE_TYPE_CLEANUP(stasis_subscription_change_type);
 	STASIS_MESSAGE_TYPE_CLEANUP(ast_multi_user_event_type);
 	aco_info_destroy(&cfg_info);
@@ -3140,7 +3062,7 @@ int stasis_init(void)
 {
 	struct stasis_config *cfg;
 	int cache_init;
-	struct ast_taskpool_options taskpool_opts = { 0, };
+	struct ast_threadpool_options threadpool_opts = { 0, };
 #ifdef AST_DEVMODE
 	struct ao2_container *subscription_stats;
 	struct ao2_container *topic_stats;
@@ -3155,21 +3077,17 @@ int stasis_init(void)
 
 	aco_option_register_custom(&cfg_info, "decline", ACO_EXACT,
 		declined_options, "", declined_handler, 0);
-	aco_option_register(&cfg_info, "minimum_size", ACO_EXACT,
-		taskpool_options, "5", OPT_INT_T, PARSE_IN_RANGE,
-		FLDSET(struct stasis_taskpool_conf, minimum_size), 0,
-		INT_MAX);
 	aco_option_register(&cfg_info, "initial_size", ACO_EXACT,
-		taskpool_options, "5", OPT_INT_T, PARSE_IN_RANGE,
-		FLDSET(struct stasis_taskpool_conf, initial_size), 0,
+		threadpool_options, "5", OPT_INT_T, PARSE_IN_RANGE,
+		FLDSET(struct stasis_threadpool_conf, initial_size), 0,
 		INT_MAX);
 	aco_option_register(&cfg_info, "idle_timeout_sec", ACO_EXACT,
-		taskpool_options, "20", OPT_INT_T, PARSE_IN_RANGE,
-		FLDSET(struct stasis_taskpool_conf, idle_timeout_sec), 0,
+		threadpool_options, "20", OPT_INT_T, PARSE_IN_RANGE,
+		FLDSET(struct stasis_threadpool_conf, idle_timeout_sec), 0,
 		INT_MAX);
 	aco_option_register(&cfg_info, "max_size", ACO_EXACT,
-		taskpool_options, "50", OPT_INT_T, PARSE_IN_RANGE,
-		FLDSET(struct stasis_taskpool_conf, max_size), 0,
+		threadpool_options, "50", OPT_INT_T, PARSE_IN_RANGE,
+		FLDSET(struct stasis_threadpool_conf, max_size), 0,
 		INT_MAX);
 
 	if (aco_process_config(&cfg_info, 0) == ACO_PROCESS_ERROR) {
@@ -3179,7 +3097,7 @@ int stasis_init(void)
 			return -1;
 		}
 
-		if (aco_set_defaults(&taskpool_option, "taskpool", default_cfg->taskpool_options)) {
+		if (aco_set_defaults(&threadpool_option, "threadpool", default_cfg->threadpool_options)) {
 			ast_log(LOG_ERROR, "Failed to initialize defaults on Stasis configuration object\n");
 			ao2_ref(default_cfg, -1);
 
@@ -3205,16 +3123,15 @@ int stasis_init(void)
 		}
 	}
 
-	taskpool_opts.version = AST_TASKPOOL_OPTIONS_VERSION;
-	taskpool_opts.minimum_size = cfg->taskpool_options->minimum_size;
-	taskpool_opts.initial_size = cfg->taskpool_options->initial_size;
-	taskpool_opts.auto_increment = 1;
-	taskpool_opts.max_size = cfg->taskpool_options->max_size;
-	taskpool_opts.idle_timeout = cfg->taskpool_options->idle_timeout_sec;
-	taskpool = ast_taskpool_create("stasis", &taskpool_opts);
+	threadpool_opts.version = AST_THREADPOOL_OPTIONS_VERSION;
+	threadpool_opts.initial_size = cfg->threadpool_options->initial_size;
+	threadpool_opts.auto_increment = 1;
+	threadpool_opts.max_size = cfg->threadpool_options->max_size;
+	threadpool_opts.idle_timeout = cfg->threadpool_options->idle_timeout_sec;
+	threadpool = ast_threadpool_create("stasis", NULL, &threadpool_opts);
 	ao2_ref(cfg, -1);
-	if (!taskpool) {
-		ast_log(LOG_ERROR, "Failed to create 'stasis-core' taskpool\n");
+	if (!threadpool) {
+		ast_log(LOG_ERROR, "Failed to create 'stasis-core' threadpool\n");
 
 		return -1;
 	}
